@@ -5,6 +5,7 @@
 
 #include "xe_pm.h"
 
+#include <linux/fault-inject.h>
 #include <linux/pm_runtime.h>
 
 #include <drm/drm_managed.h>
@@ -263,15 +264,7 @@ int xe_pm_init_early(struct xe_device *xe)
 
 	return 0;
 }
-
-static u32 vram_threshold_value(struct xe_device *xe)
-{
-	/* FIXME: D3Cold temporarily disabled by default on BMG */
-	if (xe->info.platform == XE_BATTLEMAGE)
-		return 0;
-
-	return DEFAULT_VRAM_THRESHOLD;
-}
+ALLOW_ERROR_INJECTION(xe_pm_init_early, ERRNO); /* See xe_pci_probe() */
 
 /**
  * xe_pm_init - Initialize Xe Power Management
@@ -283,7 +276,6 @@ static u32 vram_threshold_value(struct xe_device *xe)
  */
 int xe_pm_init(struct xe_device *xe)
 {
-	u32 vram_threshold;
 	int err;
 
 	/* For now suspend/resume is only allowed with GuC */
@@ -297,8 +289,7 @@ int xe_pm_init(struct xe_device *xe)
 		if (err)
 			return err;
 
-		vram_threshold = vram_threshold_value(xe);
-		err = xe_pm_set_vram_threshold(xe, vram_threshold);
+		err = xe_pm_set_vram_threshold(xe, DEFAULT_VRAM_THRESHOLD);
 		if (err)
 			return err;
 	}
@@ -682,13 +673,11 @@ void xe_pm_assert_unbounded_bridge(struct xe_device *xe)
 }
 
 /**
- * xe_pm_set_vram_threshold - Set a VRAM threshold for allowing/blocking D3Cold
+ * xe_pm_set_vram_threshold - Set a vram threshold for allowing/blocking D3Cold
  * @xe: xe device instance
- * @threshold: VRAM size in MiB for the D3cold threshold
+ * @threshold: VRAM size in bites for the D3cold threshold
  *
- * Return:
- * * 0		- success
- * * -EINVAL	- invalid argument
+ * Returns 0 for success, negative error code otherwise.
  */
 int xe_pm_set_vram_threshold(struct xe_device *xe, u32 threshold)
 {
